package net.rumati.logging.muffero.appender;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.nio.charset.Charset;
import java.util.Date;
import net.rumati.logging.muffero.layout.Layout;
import net.rumati.logging.muffero.util.FastDateFormat;

public class DatedFileAppender
        extends WriterAppender
{
    private final String prefix;
    private final String suffix;
    private final FastDateFormat dateFormat;
    private final Layout layout;
    private final Charset charset;
    private transient Writer writer = null;
    private transient String currentFileName = null;

    public DatedFileAppender(String fileNamePattern, Layout layout)
    {
        this(fileNamePattern, Charset.forName("UTF-8"), layout);
    }

    public DatedFileAppender(String fileNamePattern, Charset charset, Layout layout)
    {
        this.layout = layout;
        this.charset = charset;
        int sidx = fileNamePattern.indexOf("{");
        if (sidx < 0){
            throw new IllegalArgumentException("No date format provided");
        }
        int eidx = fileNamePattern.indexOf("}", sidx);
        if (eidx < sidx){
            throw new IllegalArgumentException("Unterminated date format");
        }
        dateFormat = FastDateFormat.getInstance(fileNamePattern.substring(sidx + 1, eidx));
        prefix = fileNamePattern.substring(0, sidx);
        suffix = fileNamePattern.substring(eidx + 1);
    }

    protected String getOutputFileName()
    {
        return prefix + dateFormat.format(new Date(System.currentTimeMillis())) + suffix;
    }

    @Override
    protected synchronized Writer getWriter()
            throws IOException
    {
        String fileName = getOutputFileName();
        if (!fileName.equals(currentFileName)){
            currentFileName = fileName;
            if (writer != null){
                try{
                    writer.close();
                }catch (IOException e){
                }
            }
            writer = new OutputStreamWriter(new FileOutputStream(fileName, true), charset);
        }
        return writer;
    }

    @Override
    protected Layout getLayout()
    {
        return layout;
    }

    public void shutdown()
    {
        if (writer != null){
            try{
                writer.close();
            }catch (IOException ignore){
            }
        }
    }
}
