package net.rumati.logging.muffero.config;

import net.rumati.logging.muffero.appender.Appender;
import java.util.LinkedList;
import java.util.List;
import net.rumati.logging.muffero.Log;
import net.rumati.logging.muffero.Logger;
import net.rumati.logging.muffero.LoggingException;
import net.rumati.logging.muffero.filter.Filter;

public abstract class Configurator
{
    private static class ConfiguredLog
    {
        private final Log log;
        private final Logger logger;

        public ConfiguredLog(Log log, Logger logger)
        {
            this.log = log;
            this.logger = logger;
        }

        public Log getLog()
        {
            return log;
        }

        public Logger getLogger()
        {
            return logger;
        }
    }

    private final List<ConfiguredLog> configuredLogs = new LinkedList<ConfiguredLog>();

    protected synchronized void addAppender(Logger logger, Appender appender, List<Filter> filters)
    {
        Log log = new Log(appender, filters);
        logger.addLog(log);
        configuredLogs.add(new ConfiguredLog(log, logger));
    }

    protected synchronized void addAppender(Logger logger, Appender appender)
    {
        addAppender(logger, appender, new LinkedList<Filter>());
    }

    public abstract void configure()
            throws LoggingException;

    public synchronized void deconfigure()
    {
        for (ConfiguredLog cl : configuredLogs){
            cl.getLogger().removeLog(cl.getLog());
        }
    }
}
