package net.rumati.logging.muffero.config.prop;

import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import net.rumati.logging.muffero.LoggingException;

public class PropertyObjectLoaderAggregator
        implements PropertyObjectLoader
{
    private final List<PropertyObjectLoader> loaders = new LinkedList<PropertyObjectLoader>();

    public void addLoader(PropertyObjectLoader loader)
    {
        loaders.add(loader);
    }

    public boolean canLoad(String className, Class<?> c)
    {
        for (PropertyObjectLoader l : loaders){
            if (l.canLoad(className, c)){
                return true;
            }
        }
        return false;
    }

    public <T> T loadObject(String className,
                            Class<T> c,
                            Map<String, String> configuration, PropertyConfigContext ctx)
            throws LoggingException
    {
        for (PropertyObjectLoader l : loaders){
            if (l.canLoad(className, c)){
                return l.loadObject(className, c, configuration, ctx);
            }
        }

        throw new LoggingException("Unable to load object type=" + c.getClass().getName() + ", class=" + className);
    }
}
