package net.rumati.logging.muffero.config.prop;

import java.util.Map;
import net.rumati.logging.muffero.LoggingException;

public abstract class SimplePropertyObjectLoader<T>
    implements PropertyObjectLoader
{
    private final Class<T> objectClass;

    public SimplePropertyObjectLoader(Class<T> objectClass)
    {
        this.objectClass = objectClass;
    }

    public final boolean canLoad(String className, Class<?> c)
    {
        if (objectClass.getName().equals(className) && c.isAssignableFrom(objectClass)){
            return true;
        }
        return false;
    }

    @SuppressWarnings("unchecked")
    public final <T> T loadObject(String className,
                            Class<T> c,
                            Map<String, String> configuration,
                            PropertyConfigContext ctx)
            throws LoggingException
    {
        if (!canLoad(className, c)){
            throw new LoggingException("Unable to load object type=" + c.getClass().getName() + ", class=" + className);
        }

        return (T)load(configuration, ctx);
    }

    public abstract T load(Map<String, String> configuration, PropertyConfigContext ctx)
        throws LoggingException;
}
