package net.rumati.logging.muffero.filter;

import java.util.List;
import net.rumati.logging.muffero.LogEvent;
import net.rumati.logging.muffero.Priority;
import net.rumati.logging.muffero.appender.Appender;

public class SimpleFilterChain
        implements FilterChain
{
    private final Filter filter;
    private final FilterChain next;

    public SimpleFilterChain(Appender appender, List<Filter> filters)
    {
        this(appender, filters, 0);
    }

    private SimpleFilterChain(Appender appender, List<Filter> filters, int index)
    {
        if (index == filters.size()){
            filter = new Filter()
            {
                public boolean isPriorityEnabled(Priority priority, FilterChain chain)
                {
                    return chain.isPriorityEnabled(priority);
                }

                public void doFilter(LogEvent event, FilterChain chain)
                {
                    chain.doFilter(event);
                }
            };
            next = new AppenderOutputFilterChain(appender);
        }else if (index == filters.size() - 1){
            filter = filters.get(index);
            next = new AppenderOutputFilterChain(appender);
        }else{
            filter = filters.get(index);
            next = new SimpleFilterChain(appender, filters, index + 1);
        }
    }

    public void doFilter(LogEvent event)
    {
        filter.doFilter(event, next);
    }

    public boolean isPriorityEnabled(Priority priority)
    {
        return filter.isPriorityEnabled(priority, next);
    }
}
